#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#------------------------------------------------------------------------------------------------
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
# IT WAS CREATED AND TESTED IN A WINDOWS ENVIRONMENT ONLY AND MAY NOT FUNCTION ON A MAC.
#------------------------------------------------------------------------------------------------
# THIS PLUGIN IS ANOTHER FIGMENT OF MY IMAGENATION AND IS NOT BASED ON OR COMPLY WITH ANYTHING! 
#------------------------------------------------------------------------------------------------
#    Name:	Rotated Box
#      By:	sdmitch
#   Usage:	Plugins>SDM Tools>CorG Tool>Rotated Box
#		 Note:	This is a modified version of Rotated Rectangle with the 3rd dimension added.
#						The default is to not add to existing components or groups, create groups, or
#						to center on base.  Any and all of these options are selectable in the WebDialog.
#    Date:	Jun 2012 Original Version.  Jun 2014 Updated and WebDialog added.
#------------------------------------------------------------------------------------------------
#
require 'sketchup'
# ------------------  MENU SETUP ---------------------- #
unless $sdm_tools_menu
	$sdm_tools_menu = UI.menu("Plugins").add_submenu("Werkzeuge von sdmitch")
	$sdm_Edge_tools = $sdm_tools_menu.add_submenu("Kanten und Linien")
	$sdm_Face_tools = $sdm_tools_menu.add_submenu("Flächen Werkzeuge")
	$sdm_CorG_tools = $sdm_tools_menu.add_submenu("Gruppen und Komponenten")
	$sdm_Misc_tools = $sdm_tools_menu.add_submenu("Diverse Werkzeuge")
end
unless file_loaded?(__FILE__)
	$sdm_CorG_tools.add_item(" Gedrehter Wuerfel") { Sketchup.active_model.select_tool SDM::Rotated_Box.new }
	file_loaded(__FILE__)
end
# ------------------------------------------------------ #
module SDM

	class Rotated_Box

		@@dlg=@@A2C=@@COB=@@GRP=nil
		
		def initialize
			@mod = Sketchup.active_model
			@ent = @mod.active_entities
			@ip = Sketchup::InputPoint.new
			@ip1 = Sketchup::InputPoint.new
			@@COB = false unless @@COB
			@@GRP = false unless @@GRP
			@@A2C = false unless @@A2C
			@guide_line = nil
			self.dialog
			self.reset
		end
		
		def dialog()
		
			unless @@dlg
				@@dlg=UI::WebDialog.new("Rotated Box",false,"RBWD",100,200,10,10,true)
				@@dlg.set_html(
					"<html>
						<body>
							<form>
								<input type='checkbox' name='a2c' #{@@A2C ? 'checked' : ""} onClick='tellSketchup(name)'> Add 2 CorG </input><br>
								<input type='checkbox' name='grp' #{@@GRP ? 'checked' : ''} onClick='tellSketchup(name)'> Group  </input>
								<input type='checkbox' name='ctr' #{@@COB ? 'checked' : ''} onClick='tellSketchup(name)'> Center </input><br>
							</form>
							<script type='text/javascript'>
								function tellSketchup(value)
								{
									window.location='skp:OptionChanged@'+value;
								}
							</script>
						</body>
					</html>"
					)
							
					RUBY_PLATFORM =~ /(darwin)/ ? @@dlg.show_modal() : @@dlg.show()
							
					@@dlg.add_action_callback("OptionChanged") {|d,p|
						
						case p
							when 'a2c' then @@A2C = !@@A2C
							when 'grp' then @@GRP = !@@GRP
							when 'ctr' then @@COB = !@@COB
						end
						
						@@dlg.close; @@dlg=nil; self.dialog; self.get_prompt
						
				}
			end
			
		end

		def reset
			@pts = []
			@bpt = []
			@state = 0
			@ip1.clear
			@height = nil
			if(@guide_line!=nil)
				Sketchup.active_model.abort_operation # in case operation aborted.
				@guide_line=nil
			end
			self.get_prompt
		end

		def onMouseMove(flags, x, y, view)
			@ip.pick view,x,y; view.tooltip = @ip.tooltip; view.refresh
			Sketchup::set_status_text @prompt, SB_PROMPT
			Sketchup::set_status_text @vcb_label, SB_VCB_LABEL
			Sketchup::set_status_text @vcb_value, SB_VCB_VALUE
		end

		def onLButtonDown(flags, x, y, view)
			case @state# Compute points
				when 0
					@bpt[0] = @ip.position
					@add2=nil;@trans=nil; ph=view.pick_helper;ph.do_pick x,y; best=ph.best_picked
					if((best.is_a?(Sketchup::Group)||best.is_a?(Sketchup::ComponentInstance)) && @@A2C)
						(@add2=best;@trans=best.transformation) if best.is_a?(Sketchup::Group)
						(@add2=best.definition;@trans=best.transformation) if best.is_a?(Sketchup::ComponentInstance)
					end
					@state = 1; @ip1.copy! @ip; self.get_prompt
				when 1
					@bpt[1] = @ip.position
					@width = @bpt[0].distance @bpt[1]
					@state = 2; @ip1.clear; self.get_prompt
				when 2
					pt1 = @ip.position
					pt2 = pt1.project_to_line @bpt
					@vec = pt1 - pt2
					@depth = @vec.length
					if( @depth > 0 )
						square_point = pt2.offset(@vec, @width)# test for a square
						if( view.pick_helper.test_point(square_point, x, y) )
							@depth = @width
							@bpt[2] = @bpt[1].offset(@vec, @depth)
							@bpt[3] = @bpt[0].offset(@vec, @depth)
							view.tooltip = "Square"
						else
							@bpt[2] = @bpt[1].offset(@vec)
							@bpt[3] = @bpt[0].offset(@vec)
						end
					else
						@bpt[2] = @bpt[1]
						@bpt[3] = @bpt[0]
					end
					@ip1.clear
					v1=(@bpt[1]-@bpt[2]); v2=(@bpt[3]-@bpt[2]); @normal= v1.cross(v2)
					@mod.start_operation "Rotated Box"
					@guide_line = @ent.add_cline(@ip.position,@normal)
					@state = 3; self.get_prompt
				when 3
					if( @height != 0.0 )
						self.create_box
					else
						UI.beep; puts "Zero height"
					end
			 end
			view.lock_inference
		end
		
		def make_box(g,w,d,h)
			# modified code from Rotated Rectangel Tool modified by Chris Phillips
			pts = [[-(w/2),-(d/2),-(h/2)],[w/2,-(d/2),-(h/2)],[w/2,d/2,-(h/2)],[-(w/2),d/2,-(h/2)],[-(w/2),-(d/2),-(h/2)]]
			base = g.entities.add_face pts
			h = -h if base.normal.dot(Z_AXIS) < 0.0
			base.pushpull h
		end
		
		def create_box
			@guide_line.erase!
			if @add2
				grp=@add2.entities.add_group;
				@pts.each{|p| p.transform! @trans.inverse}
			else
				grp=@ent.add_group
			end
			make_box(grp,@width,@depth,@height)
			v1=(@pts[1]-@pts[0]);v2=(@pts[3]-@pts[0]);v3=(@pts[4]-@pts[0])
			v1.reverse! if(v1.cross(v2).dot(v3)<0)
			vc=(@pts[2]-@pts[0]);vc.length=vc.length/2;@center=@pts[0]+vc
			xform=Geom::Transformation.new(v1,v2,v3,@center)*Geom::Transformation.new([0,0,@height/2])
			grp.transform!(xform);#orient the group axis to match the group side directions
			grp.explode unless @@GRP
			@mod.commit_operation
			self.reset
		end
		
		def onRButtonDown(flag, x, y, view)
			@state>0 ? self.reset : onCancel(flag,view)
		end

		def onCancel(flag,view)
			Sketchup.send_action "selectSelectionTool:"
		end

		def deactivate(view)
			@@dlg.close; @@dlg=nil
		end
		
		def onUserText(text, view)
			begin
				value = text.to_l
			rescue
				value = nil; UI.beep; # Error parsing the text
				Sketchup::set_status_text "", SB_VCB_VALUE
			end
			return if !value
			case @state
				when 1
					vec = @bpt[1] - @bpt[0]
					if( vec.length > 0.0 )
						@width = value# update the width
						@bpt[1] = @bpt[0].offset(vec,@width)
						@state += 1; self.get_prompt
						draw(view)
					end
				when 2
					@vec = @bpt[3] - @bpt[0]
					if( @vec.length > 0.0 )
						@depth = value# update the depth
						@bpt[2] = @bpt[1].offset(@vec,@depth)
						@bpt[3] = @bpt[0].offset(@vec,@depth)
						v1=(@bpt[1]-@bpt[2]); v2=(@bpt[3]-@bpt[2]); @normal= v1.cross(v2)
						@mod.start_operation "Rotated Box"
						@guide_line = @ent.add_cline(@bpt[2],@normal)
						@state += 1; self.get_prompt
						draw(view)
					end
				when 3
					if value > 0.0
						@height = value# update the height
						@vcb_label = "Height:"; @vcb_value = @height.to_s
						if @@COB
							for i in 0..3
								@pts[i] = @bpt[i].offset(@vec,-@height/2)
								@pts[i+4] = @pts[i].offset(@vec)
							end
						else
							for i in 0..3
								@pts[i] = @bpt[i]
								@pts[i+4] = @pts[i].offset(@vec)
							end
						end
						self.create_box
					end
			end
		end

		def draw(view)
			if( @ip.valid? && @ip.display? )
				@ip.draw(view)# Show the current input point
			end
			if @state == 1# draw line from the start to current point
				@bpt[1]=@ip.position; @width = @bpt[0].distance @bpt[1]
				if @width > 0
					@vcb_label = "Width:"; @vcb_value = @width.to_s
					view.set_color_from_line(@ip1, @ip); 
					view.draw(GL_LINE_STRIP, @bpt[0], @bpt[1])
				end
			elsif @state == 2	# draw the base rectangle
				pt1 = @ip.position; pt2 = pt1.project_to_line @bpt
				@vec = pt1 - pt2; @depth = @vec.length
				if @depth > 0 
					@vcb_label = "Depth:"; @vcb_value = @depth.to_s; 
					@bpt[2]=@bpt[1].offset(@vec)
					@bpt[3]=@bpt[0].offset(@vec)
					view.drawing_color = "blue"
					view.draw(GL_LINE_LOOP, @bpt)
					if @depth==@width
						view.line_stipple = "-"
						view.draw_line @bpt[0],@bpt[2]
					end
				end
			elsif @state == 3	# draw the box
				view.drawing_color = "red"
				view.draw GL_LINE_LOOP, @bpt[0..3]
				pt1 = @ip.position
				pln = Geom.fit_plane_to_points @bpt[0],@bpt[1],@bpt[2]
				pt2 = pt1.project_to_plane pln
				@vec = pt1 - pt2; 
				if  @vec.length > 0
					@height = @vec.length
					@vcb_label = "Height:"; @vcb_value = @height.to_s
					if @@COB
						for i in 0..3
							@pts[i] = @bpt[i].offset(@vec,-@height/2)
							@pts[i+4] = @pts[i].offset(@vec)
						end
					else
						for i in 0..3
							@pts[i] = @bpt[i]
							@pts[i+4] = @pts[i].offset(@vec)
						end
					end
					view.draw GL_LINE_LOOP, @pts[0..3]
					view.draw GL_LINE_LOOP, @pts[4..7]
					for i in 0..3
						view.draw GL_LINES,@pts[i],@pts[i+4]
					end
				end
			 end
		end

		def get_prompt
			case @state
				when 0 ; prompt = "Pick first point, "; atm="Right click in space or Esc to exit."
				when 1 ; prompt = "Pick second point or keyin desired width, "
				when 2 ; prompt = "Pick third point or keyin desired depth, "
				when 3 ; prompt = "Pick forth point or keyin desired height, "
			end
			if [1,2,3].include? @state then atm="Right click in space to start over, Esc to exit." end
			@prompt = prompt+atm; 
		end
		
	end # of class RotatedBox

end
